-- Power bar module

local powertype
local powcolor = {}
local predictBase = 14
local newPredict
local adj = 0

function SN:SetupPower()
	
	local f = SN.F
	local v = SN.Media

	SN:GetPowerColor()

	f.powercontainer = CreateFrame("Frame", nil, UIParent)
	f.powercontainer:SetPoint("CENTER", UIParent, "CENTER", 0, 100)
	f.powercontainer:SetSize(250, 22)
	f.powercontainer:SetMovable(true)
	f.powercontainer:EnableMouse(true)
	f.powercontainer:SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" and not self.isMoving then
			self:StartMoving()
			self.isMoving = true
		end
	end)
	f.powercontainer:SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" and self.isMoving then
			self:StopMovingOrSizing()
			self.isMoving = false
		end
	end)
	f.powercontainer:RegisterEvent("UNIT_DISPLAYPOWER", "UpdatePowerType")


	--The actual power bar
	f.power = CreateFrame("StatusBar", nil, f.powercontainer)
	f.power:SetPoint("CENTER", f.powercontainer, "CENTER", 0, 0)
	f.power:SetSize(250, 22)
	f.power:SetStatusBarTexture(v.media.minTex)	
	f.power:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
	f.power:SetMinMaxValues(0, (UnitPowerMax("player") > 0) and UnitPowerMax("player") or 100)	
	f.power:SetValue(UnitPower("player"))
	f.power.TimeSinceLastUpdate = 0
	f.power:SetScript("OnUpdate", UpdatePower)
	f.power:SetBackdrop({
				bgFile = v.media.blankTex,
			})
	f.power:SetBackdropColor(0.1, 0.1, 0.1, 0)
	f.power:SetScript("OnEvent", function(self, event, ...)
			local sourceName, spellName, _, _, spellId = ...
			
			if event == "UNIT_SPELLCAST_START" then
				if sourceName == "player" then						
					if spellId  == 56641 or spellId == 77767 then -- HUNTER: Steady Shot / Cobra Shot
						SN:updatePredict()
					end
				end
			elseif event == "UNIT_SPELLCAST_STOP" or event == "UNIT_SPELLCAST_FAILED" then
				if spellId == 56641 or spellId == 77767 then -- HUNTER: Steady Shot / Cobra Shot							
					self.predict:Hide()
				end
			end
		end)

	--The power text
	f.power.value = f.power:CreateFontString(nil, "OVERLAY")
	f.power.value:SetPoint("CENTER", f.power, "CENTER", 4, 0)
	f.power.value:SetFont(v.media.font, 14, "OUTLINE")
	f.power.value:SetJustifyH("CENTER")
	f.power.value:SetShadowOffset(1, -1)
	f.power.value:SetTextColor(0, 1, 0)
	f.power.value:SetText(UnitPower("player"))
	
	--SN:MakeSmooth(f.power)

	f.power.predict = CreateFrame("StatusBar", nil, f.power)			
	f.power.predict:SetStatusBarTexture(f.power:GetStatusBarTexture():GetTexture()) -- Use the main bar's texture
	f.power.predict:SetMinMaxValues(0, 1)
	f.power.predict:SetValue(1)
	f.power.predict:SetFrameLevel(f.power:GetFrameLevel())
	f.power.predict:SetSize((f.power:GetWidth() / 100) * (predictBase + SN:getPredictAdj()), f.power:GetHeight())
	
	SN:SNCreateBG(f.powercontainer)
	-- Register Events to support the bar
	f.power:RegisterEvent("UNIT_SPELLCAST_START")
	f.power:RegisterEvent("UNIT_SPELLCAST_STOP")
	f.power:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	f.power:RegisterEvent("UNIT_SPELLCAST_FAILED")
	f.power:RegisterEvent("UNIT_MAXPOWER")
	f.power:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	f.power:RegisterEvent("UNIT_POWER")

	SN:PowebarTicks()
end

function SN:GetPowerColor()
	
	powertype = select(2, UnitPowerType("player"))
	if powertype == "MANA" then
		powcolor = SN.Media.power.MANA
	elseif powertype == "RAGE" then
		powcolor = SN.Media.power.RAGE
	elseif powertype == "FOCUS" then
		powcolor = SN.Media.power.FOCUS
	elseif powertype == "ENERGY" then
		powcolor = SN.Media.power.ENERGY
	elseif powertype == "RUNIC_POWER" then
		powcolor = SN.Media.power.RUNIC_POWER
	end
	return powcolor
	
end

function UpdatePower(frame, elapsed)
	frame:SetValue(UnitPower("player"))
	frame:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
	frame.value:SetText(UnitPower("player"))

	frame.TimeSinceLastUpdate = frame.TimeSinceLastUpdate + elapsed
	if frame.TimeSinceLastUpdate > SN_UpdateInterval then
		--frame:SetValue(UnitPower("player"))
		--frame:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
		--frame.value:SetText(UnitPower("player"))
		frame.TimeSinceLastUpdate = 0

		-- Handle indicator tick marks
		local _, name, cost -- removed to prevent local re-creation
		for jj=1, 2 do
			if (SN.V.profile.powerbar.hunter.survival.ticks[jj][1] == true) and (frame["tick"..jj] ~= nil) 
				and (SN.V.profile.powerbar.hunter.survival.ticks[1][1] == true) then
				
				if (jj > 1) then
					name, _, _, cost = GetSpellInfo(SN.V.profile.powerbar.hunter.survival.ticks[jj][2])
				end
					
				if (jj == 1) or ( (jj > 1) and name and cost and (cost > 0) ) then
					if jj == 1 then
						frame["tick"..jj]:ClearAllPoints()
						frame["tick"..jj]:SetPoint("LEFT", frame, "LEFT", SN:GetSpecSpellCost() * (frame:GetWidth() / select(2, frame:GetMinMaxValues() ) ) - 5, 0)
					else
						frame["tick"..jj]:ClearAllPoints()
						frame["tick"..jj]:SetPoint("LEFT", frame, "LEFT", 
							( (SN.V.profile.powerbar.hunter.survival.ticks[jj][3] == true) and (SN:GetSpecSpellCost() + cost) or cost ) * 
							(frame:GetWidth() / select(2, frame:GetMinMaxValues() ) ) - 5, 0)
					end
					frame["tick"..jj]:Show()
				else
					frame["tick"..jj]:Hide()
				end
			end
		end
	end
end

function SN:UpdatePowerType()
	SN:GetPowerColor()
	SN.F.power:SetValue(UnitPower("player"))
	SN.F.power:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
	SN.F.power.value:SetText(UnitPower("player"))
end

local function Smooth(self, value)
	if (value ~= self:GetValue()) or (value == 0) then
		self.smoothing = value
	else
		self.smoothing = nil
	end
end

function SN:MakeSmooth(powerframe)
	if powerframe.SetValue_ORI then return end
	powerframe.SetValue_ORI = powerframe.SetValue
	powerframe.SetValue = Smooth
	
	powerframe.smoother = CreateFrame("Frame", nil, powerframe)
	local min, max = math.min, math.max
	powerframe.smoother:SetScript("OnUpdate", function(self)
		local rate = GetFramerate()
		local limit = 30 / rate
		if self:GetParent().smoothing then
			local cur = self:GetParent():GetValue()
			local new = cur + min((self:GetParent().smoothing - cur) / 3, max(self:GetParent().smoothing - cur, limit))
			self:GetParent():SetValue_ORI(new)
			if cur == self:GetParent().smoothing or abs(new - self:GetParent().smoothing) < 2 then
				self:GetParent():SetValue_ORI(self:GetParent().smoothing)
				self:GetParent().smoothing = nil
			end
		end
	end)
end

function SN:RemoveSmooth(powerframe)
	if not powerframe.SetValue_ORI then return end
	
	powerframe.smoother:SetScript("OnUpdate", nil)
	powerframe.smoother:SetParent(nil)
	powerframe.smoother = nil
	
	powerframe.SetValue = powerframe.SetValue_ORI
	powerframe.SetValue_ORI = nil
end

function SN:PowebarTicks()
	local name, cost
	for i=1, 2 do
		if (SN.V.profile.powerbar.hunter.survival.ticks[i][1] == true) and (SN.V.profile.powerbar.hunter.survival.ticks[i][6] == 0)
			and (SN.V.profile.powerbar.hunter.survival.ticks[1][1] == true) then
				
			local _, _name, _cost
			if i > 1 then
				name, _, _, cost = GetSpellInfo(SN.V.profile.powerbar.hunter.survival.ticks[i][2])
			end
				
			if (i == 1) or ( (i > 1) and name and cost and (cost > 0) ) then
				SN.F.power["tick"..i] = CreateFrame("Frame", "SN_POWER_TICKS"..i, SN.F.power)
				SN.F.power["tick"..i]:SetSize(10, SN.F.power:GetHeight() * 1.6)
				SN.F.power["tick"..i].tex = SN.F.power["tick"..i]:CreateTexture(nil, "OVERLAY")
				SN.F.power["tick"..i].tex:SetAllPoints(SN.F.power["tick"..i])
				SN.F.power["tick"..i].tex:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")	
				SN.F.power["tick"..i].tex:SetBlendMode("ADD")
				--SN.F.power["indicatorTick"..i]:SetAlpha(1)
				--SN.F.power["indicatorTick"..i]:Hide()
			end
		end
	end
end

function SN:GetSpecSpellCost()
	if select(2, UnitClass("player")) == "HUNTER" then
		currentCost = select(4, GetSpellInfo(SN.HunterSpell[GetSpecialization() or 1]) )
	end
	if (currentCost ~= 0) then
		lastCost = currentCost
	end
	return lastCost
end

function SN:getPredictAdj()
	adj = 0
	
	if (GetSpecialization() == 2) and select(1, UnitAura("player", GetSpellInfo(53220), nil, "HELPFUL") ) then -- 53220 = Steady Focus (buff)
		adj = adj + 3
	end
	
	return (adj)
end

function SN:updatePredict()

	-- Add values if AotF active and talent points, and Termination points
	newPredict = predictBase + SN:getPredictAdj()
		
	SN.F.power.predict:SetSize((SN.F.power:GetWidth() / 100) * newPredict, SN.F.power:GetHeight() )

	SN.F.power.predict:ClearAllPoints()
	SN.F.power.predict:SetPoint("LEFT", SN.F.power, "LEFT",	SN.F.power:GetWidth() / (select(2, SN.F.power:GetMinMaxValues() ) / SN.F.power:GetValue() ), 0)

	if (UnitPower("player") + newPredict) > UnitPowerMax("player") then
		SN.F.power.predict:SetSize((SN.F.power:GetWidth() / 100) * (UnitPowerMax("player") - UnitPower("player") ), SN.F.power:GetHeight() )			
	end
		
	SN.F.power.predict:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
		
	if ( (UnitPowerMax("player") - UnitPower("player") ) > 0) and (not UnitIsDeadOrGhost("player") ) then
		SN.F.power.predict:SetAlpha(SN.F.power:GetAlpha() * 0.6)
		SN.F.power.predict:Show()
	else
		SN.F.power.predict:Hide()
	end
end

function SN:InitPower()
	-- setup the power bar
	local unitclass = select(2, UnitClass("player"))
	if  unitclass == "HUNTER" or unitclass == "ROGUE" or (unitclass == "DRUID" and GetSpecialization() == 3 or GetSpecialization() == 2) then
		
		SN:SetupPower()
	end
end